@@ -22,6 +22,8 @@
 
 """A simple colormap module."""
 
+import warnings
+
 import numpy as np
 from trollimage.colorspaces import rgb2hcl, hcl2rgb
 
@@ -132,7 +134,7 @@ def palettize(arr, colors, values):
 
 def _palettize_dask(darr, colors, values):
     """Apply a palette to a dask array."""
-    return darr.map_blocks(_palettize, values, dtype=colors.dtype)
+    return darr.map_blocks(_palettize, values, dtype=int)
 
 
 def _palettize(arr, values):
@@ -155,6 +157,18 @@ def _digitize_array(arr, values):
 class Colormap(object):
     """The colormap object.
 
+    Args:
+        *args: Series of (value, color) tuples. These positional arguments
+            are only used if the ``values`` and ``colors`` keyword arguments
+            aren't provided.
+        values: One dimensional array-like of control points where
+            each corresponding color is applied. Must be the same number of
+            elements as colors and must be monotonically increasing.
+        colors: Two dimensional array-like of RGB or RGBA colors where each
+            color is applied to a specific control point. Must be the same
+            number of colors as control points (values). Colors should be
+            floating point numbers between 0 and 1.
+
     Initialize with tuples of (value, (colors)), like this::
 
       Colormap((-75.0, (1.0, 1.0, 0.0)),
@@ -162,7 +176,6 @@ class Colormap(object):
                (-40.0, (1, 1, 1)),
                (30.0, (0, 0, 0)))
 
-
     You can also concatenate colormaps together, try::
 
       cm = cm1 + cm2
@@ -174,11 +187,26 @@ class Colormap(object):
         if 'colors' in kwargs and 'values' in kwargs:
             values = kwargs['values']
             colors = kwargs['colors']
+        elif 'colors' in kwargs or 'values' in kwargs:
+            raise ValueError("Both 'colors' and 'values' must be provided.")
         else:
             values = [a for (a, b) in tuples]
             colors = [b for (a, b) in tuples]
         self.values = np.array(values)
-        self.colors = np.array(colors)
+        self.colors = self._validate_colors(colors)
+        if self.values.shape[0] != self.colors.shape[0]:
+            raise ValueError("'values' and 'colors' should have the same "
+                             "number of elements. Got "
+                             f"{self.values.shape[0]} and {self.colors.shape[0]}.")
+
+    def _validate_colors(self, colors):
+        colors = np.array(colors)
+        if colors.ndim != 2 or colors.shape[-1] not in (3, 4):
+            raise ValueError("Colormap 'colors' must be RGB or RGBA. Got unexpected shape: {}".format(colors.shape))
+        if not np.issubdtype(colors.dtype, np.floating):
+            warnings.warn("Colormap 'colors' should be flotaing point numbers between 0 and 1.")
+            colors = colors.astype(np.float64)
+        return colors
 
     def colorize(self, data):
         """Colorize a monochromatic array *data*, based on the current colormap."""
@@ -188,30 +216,119 @@ class Colormap(object):
         """Palettize a monochromatic array *data* based on the current colormap."""
         return palettize(data, self.colors, self.values)
 
+    def to_rgb(self):
+        """Return colormap with RGB colors.
+
+        If already RGB then the same instance is returned.
+        If an Alpha channel exists in the colormap, it is dropped.
+
+        """
+        if self.colors.shape[-1] == 3:
+            return self
+
+        values = self.values.copy()
+        colors = self.colors.copy()
+        return Colormap(
+            values=values,
+            colors=colors[:, :3]
+        )
+
+    def to_rgba(self):
+        """Return colormap with RGBA colors.
+
+        If already RGBA then the same instance is returned.
+        If not already RGBA, a completely opaque (1.0) color
+
+        """
+        if self.colors.shape[-1] == 4:
+            return self
+
+        values = self.values.copy()
+        colors = np.empty((self.colors.shape[0], 4), dtype=self.colors.dtype)
+        colors[:, :3] = self.colors
+        colors[:, 3] = 1.0
+        return Colormap(
+            values=values,
+            colors=colors
+        )
+
     def __add__(self, other):
         """Append colormap together."""
-        new = Colormap()
-        new.values = np.concatenate((self.values, other.values))
-        new.colors = np.concatenate((self.colors, other.colors))
-        return new
+        old, other = self._normalize_color_arrays(self, other)
+        values = np.concatenate((old.values, other.values))
+        if not (np.diff(values) >= 0).all():
+            raise ValueError("Merged colormap 'values' are not monotonically "
+                             "increasing or equal.")
+        colors = np.concatenate((old.colors, other.colors))
+        return Colormap(
+            values=values,
+            colors=colors,
+        )
 
-    def reverse(self):
-        """Reverse the current colormap in place."""
-        self.colors = np.flipud(self.colors)
+    @staticmethod
+    def _normalize_color_arrays(cmap1, cmap2):
+        num_bands1 = cmap1.colors.shape[-1]
+        num_bands2 = cmap2.colors.shape[-1]
+        if num_bands1 == num_bands2:
+            return cmap1, cmap2
+        return cmap1.to_rgba(), cmap2.to_rgba()
 
-    def set_range(self, min_val, max_val):
-        """Set the range of the colormap to [*min_val*, *max_val*]."""
+    def reverse(self, inplace=True):
+        """Reverse the current colormap in place.
+
+        Args:
+            inplace (bool): If True (default), modify the colors of this
+                Colormap inplace. If False, return a new instance.
+
+        """
+        colors = np.flipud(self.colors)
+        if not inplace:
+            return Colormap(
+                values=self.values.copy(),
+                colors=colors
+            )
+        self.colors = colors
+        return self
+
+    def set_range(self, min_val, max_val, inplace=True):
+        """Set the range of the colormap to [*min_val*, *max_val*].
+
+        If min is greater than max then the Colormap's colors are reversed
+        before values are updated to the new range. This is done because
+        Colormap ``values`` must always be monotonically increasing.
+
+        Args:
+            min_val (float): New minimum value for the control points in
+                this colormap.
+            max_val (float): New maximum value for the control points in
+                this colormap.
+            inplace (bool): If True (default), modify the values inplace.
+                If False, return a new Colormap instance.
+
+        """
         if min_val > max_val:
+            cmap = self.reverse(inplace=inplace)
             max_val, min_val = min_val, max_val
-        self.values = (((self.values * 1.0 - self.values.min()) /
-                        (self.values.max() - self.values.min()))
-                       * (max_val - min_val) + min_val)
+        else:
+            cmap = self
+
+        values = (((cmap.values * 1.0 - cmap.values.min()) /
+                   (cmap.values.max() - cmap.values.min()))
+                  * (max_val - min_val) + min_val)
+        if not inplace:
+            return Colormap(
+                values=values,
+                colors=cmap.colors.copy()
+            )
+
+        cmap.values = values
+        return cmap
 
     def to_rio(self):
         """Convert the colormap to a rasterio colormap."""
-        self.colors = (((self.colors * 1.0 - self.colors.min()) /
-                        (self.colors.max() - self.colors.min())) * 255)
-        return dict(zip(self.values, tuple(map(tuple, self.colors))))
+        colors = (((self.colors * 1.0 - self.colors.min()) /
+                   (self.colors.max() - self.colors.min())) * 255)
+        return dict(zip(self.values, tuple(map(tuple, colors))))
 
 
 # matlab jet "#00007F", "blue", "#007FFF", "cyan", "#7FFF7F", "yellow",
