@@ -1,26 +1,21 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright (c) 2009-2015, 2017.
-
-# Author(s):
-
-#   Martin Raspaud <martin.raspaud@smhi.se>
-#   Adam Dybbroe <adam.dybbroe@smhi.se>
-
-# This file is part of mpop.
-
-# mpop is free software: you can redistribute it and/or modify it
+# Copyright (c) 2009-2021 trollimage developers
+#
+# This file is part of trollimage.
+#
+# trollimage is free software: you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
-
-# mpop is distributed in the hope that it will be useful, but
+#
+# trollimage is distributed in the hope that it will be useful, but
 # WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
-# along with mpop.  If not, see <http://www.gnu.org/licenses/>.
+# along with trollimage.  If not, see <http://www.gnu.org/licenses/>.
 """Module for testing the image and xrimage modules."""
 import os
 import random
@@ -1332,6 +1327,27 @@ class TestXRImage:
             np.testing.assert_allclose(file_data[2], exp[:, :, 2])
             np.testing.assert_allclose(file_data[3], 255)
 
+    @pytest.mark.skipif(sys.platform.startswith('win'), reason="'NamedTemporaryFile' not supported on Windows")
+    def test_save_cloud_optimized_geotiff(self):
+        """Test saving cloud optimized geotiffs."""
+        import xarray as xr
+        from trollimage import xrimage
+        import rasterio as rio
+
+        # trigger COG driver to create 2 overview levels
+        # COG driver is only available in GDAL 3.1 or later
+        if rio.__gdal_version__ >= '3.1':
+            data = xr.DataArray(np.arange(1200*1200*3).reshape(1200, 1200, 3), dims=[
+                'y', 'x', 'bands'], coords={'bands': ['R', 'G', 'B']})
+            img = xrimage.XRImage(data)
+            assert np.issubdtype(img.data.dtype, np.integer)
+            with NamedTemporaryFile(suffix='.tif') as tmp:
+                img.save(tmp.name, tiled=True, overviews=[], driver='COG')
+                with rio.open(tmp.name) as f:
+                    # The COG driver should add a tag indicating layout
+                    assert(f.tags(ns='IMAGE_STRUCTURE')['LAYOUT'] == 'COG')
+                    assert len(f.overviews(1)) == 2
+
     @pytest.mark.skipif(sys.platform.startswith('win'), reason="'NamedTemporaryFile' not supported on Windows")
     def test_save_overviews(self):
         """Test saving geotiffs with overviews."""
@@ -1549,16 +1565,29 @@ class TestXRImage:
 
         np.testing.assert_allclose(img.data.values, res, atol=1.e-6)
 
-    def test_logarithmic_stretch(self):
+    @pytest.mark.parametrize(
+        ("min_stretch", "max_stretch"),
+        [
+            (None, None),
+            ([0.0, 1.0 / 74.0, 2.0 / 74.0], [72.0 / 74.0, 73.0 / 74.0, 1.0]),
+        ]
+    )
+    @pytest.mark.parametrize("base", ["e", "10", "2"])
+    def test_logarithmic_stretch(self, min_stretch, max_stretch, base):
         """Test logarithmic strecthing."""
         import xarray as xr
         from trollimage import xrimage
+        from .utils import assert_maximum_dask_computes
 
         arr = np.arange(75).reshape(5, 5, 3) / 74.
         data = xr.DataArray(arr.copy(), dims=['y', 'x', 'bands'],
                             coords={'bands': ['R', 'G', 'B']})
-        img = xrimage.XRImage(data)
-        img.stretch(stretch='logarithmic')
+        with assert_maximum_dask_computes(0):
+            img = xrimage.XRImage(data)
+            img.stretch(stretch='logarithmic',
+                        min_stretch=min_stretch,
+                        max_stretch=max_stretch,
+                        base=base)
         enhs = img.data.attrs['enhancement_history'][0]
         assert enhs == {'log_factor': 100.0}
         res = np.array([[[0., 0., 0.],
