@@ -372,8 +372,28 @@ class XRImage(object):
         """Mode of the image."""
         return ''.join(self.data['bands'].values)
 
+    @staticmethod
+    def _gtiff_to_cog_kwargs(format_kwargs):
+        """Convert GDAL Geotiff driver options to COG driver options.
+
+        The COG driver automatically sets some format options
+        but zlevel is called level and blockxsize is called blocksize.
+        Convert kwargs to save() from GTiff driver to COG driver.
+
+        """
+        format_kwargs.pop('photometric', None)
+        if 'zlevel' in format_kwargs:
+            format_kwargs['level'] = format_kwargs.pop('zlevel')
+        if 'jpeg_quality' in format_kwargs:
+            format_kwargs['quality'] = format_kwargs.pop('jpeg_quality')
+        format_kwargs.pop('tiled', None)
+        if 'blockxsize' in format_kwargs:
+            format_kwargs['blocksize'] = format_kwargs.pop('blockxsize')
+        format_kwargs.pop('blockysize', None)
+        return format_kwargs
+
     def save(self, filename, fformat=None, fill_value=None, compute=True,
-             keep_palette=False, cmap=None, **format_kwargs):
+             keep_palette=False, cmap=None, driver=None, **format_kwargs):
         """Save the image to the given *filename*.
 
         Args:
@@ -386,6 +406,12 @@ class XRImage(object):
                            If the format allows, geographical information will
                            be saved to the ouput file, in the form of grid
                            mapping or ground control points.
+            driver (str): can override the choice of rasterio/gdal driver
+                        which is normally selected from the filename or fformat.
+                        This is an implementation detail normally avoided but
+                        can be necessary if you wish to distinguish between
+                        GeoTIFF drivers ("GTiff" is the default, but you can
+                        specify "COG" to write a Cloud-Optimized GeoTIFF).
             fill_value (float): Replace invalid data values with this value
                                 and do not produce an Alpha band. Default
                                 behavior is to create an alpha band.
@@ -420,7 +446,7 @@ class XRImage(object):
         fformat = fformat or kwformat or os.path.splitext(filename)[1][1:]
         if fformat in ('tif', 'tiff', 'jp2') and rasterio:
 
-            return self.rio_save(filename, fformat=fformat,
+            return self.rio_save(filename, fformat=fformat, driver=driver,
                                  fill_value=fill_value, compute=compute,
                                  keep_palette=keep_palette, cmap=cmap,
                                  **format_kwargs)
@@ -433,6 +459,7 @@ class XRImage(object):
                  overviews_minsize=256, overviews_resampling=None,
                  include_scale_offset_tags=False,
                  scale_offset_tags=None,
+                 driver=None,
                  **format_kwargs):
         """Save the image using rasterio.
 
@@ -440,6 +467,10 @@ class XRImage(object):
             filename (string): The filename to save to.
             fformat (string): The format to save to. If not specified (default),
                 it will be infered from the file extension.
+            driver (string): The gdal driver to use. If not specified (default),
+                it will be inferred from the fformat, but you can override the
+                default GeoTIFF driver ("GTiff") with "COG" if you want to create
+                a Cloud_Optimized GeoTIFF (and set `tiled=True,overviews=[]`).
             fill_value (number): The value to fill the missing data with.
                 Default is ``None``, translating to trying to keep the data
                 transparent.
@@ -457,7 +488,9 @@ class XRImage(object):
 
                 If provided as an empty list, then levels will be
                 computed as powers of two until the last level has less
-                pixels than `overviews_minsize`.
+                pixels than `overviews_minsize`.  If driver='COG' then use
+                `overviews=[]` to get a Cloud-Optimized GeoTIFF with a correct
+                set of overviews created automatically.
                 Default is to not add overviews.
             overviews_minsize (int): Minimum number of pixels for the smallest
                 overview size generated when `overviews` is auto-generated.
@@ -484,7 +517,13 @@ class XRImage(object):
                    'tif': 'GTiff',
                    'tiff': 'GTiff',
                    'jp2': 'JP2OpenJPEG'}
-        driver = drivers.get(fformat, fformat)
+        # If fformat is specified but not driver then convert it into a driver
+        driver = driver or drivers.get(fformat, fformat)
+        # The COG driver adds overviews so we don't need to create them ourself.
+        # One thing we can't do is prevent any overviews, if we use None then
+        # the COG driver will create automatically, we can't pass OVERVIEWS=NONE.
+        if driver == 'COG' and overviews == []:
+            overviews = None
         if include_scale_offset_tags:
             warnings.warn(
                 "include_scale_offset_tags is deprecated, please use "
@@ -502,7 +541,7 @@ class XRImage(object):
         crs = None
         gcps = None
         transform = None
-        if driver in ['GTiff', 'JP2OpenJPEG']:
+        if driver in ['COG', 'GTiff', 'JP2OpenJPEG']:
             if not np.issubdtype(data.dtype, np.floating):
                 format_kwargs.setdefault('compress', 'DEFLATE')
             photometric_map = {
@@ -554,6 +593,10 @@ class XRImage(object):
             scale, offset = self.get_scaling_from_history(data.attrs.get('enhancement_history', []))
             tags[scale_label], tags[offset_label] = invert_scale_offset(scale, offset)
 
+        # If we are changing the driver then use appropriate kwargs
+        if driver == 'COG':
+            format_kwargs = self._gtiff_to_cog_kwargs(format_kwargs)
+
         # FIXME add metadata
         r_file = RIOFile(filename, 'w', driver=driver,
                          width=data.sizes['x'], height=data.sizes['y'],
@@ -1279,34 +1322,69 @@ class XRImage(object):
                                   axis=self.data.dims.index('bands'))
         self.data.attrs.setdefault('enhancement_history', []).append({'hist_equalize': True})
 
-    def stretch_logarithmic(self, factor=100.):
-        """Move data into range [1:factor] through normalized logarithm."""
+    def stretch_logarithmic(self, factor=100., base="e", min_stretch=None, max_stretch=None):
+        """Move data into range [1:factor] through normalized logarithm.
+
+        Args:
+            factor (float): Maximum of the range data will be scaled to
+                before applying the log function. Image data will be scaled
+                to a 1 to ``factor`` range.
+            base (str): Type of log to use. Defaults to natural log ("e"),
+                but can also be "10" for base 10 log or "2" for base 2 log.
+            min_stretch (float or list): Minimum input value to scale from.
+                Data will be clipped to this value before being scaled to
+                the 1:factor range. By default (None), the limits are computed
+                on the fly but with a performance penalty. May also be a list
+                for multi-band images.
+            max_stretch (float or list): Maximum input value to scale from.
+                Data will be clipped to this value before being scaled to
+                the 1:factor range. By default (None), the limits are computed
+                on the fly but with a performance penalty. May also be a list
+                for multi-band images.
+
+        """
         logger.debug("Perform a logarithmic contrast stretch.")
         crange = (0., 1.0)
+        log_func = np.log if base == "e" else getattr(np, "log" + base)
+        min_stretch, max_stretch = self._convert_log_minmax_stretch(min_stretch, max_stretch)
 
-        b__ = float(crange[1] - crange[0]) / np.log(factor)
+        b__ = float(crange[1] - crange[0]) / log_func(factor)
         c__ = float(crange[0])
 
-        def _band_log(arr):
-            slope = (factor - 1.) / float(arr.max() - arr.min())
-            arr = 1. + (arr - arr.min()) * slope
-            arr = c__ + b__ * da.log(arr)
+        def _band_log(arr, min_input, max_input):
+            slope = (factor - 1.) / (max_input - min_input)
+            arr = np.clip(arr, min_input, max_input)
+            arr = 1. + (arr - min_input) * slope
+            arr = c__ + b__ * log_func(arr)
             return arr
 
         band_results = []
-        for band in self.data['bands'].values:
+        for band_idx, band in enumerate(self.data['bands'].values):
             if band == 'A':
                 continue
             band_data = self.data.sel(bands=band)
-            res = _band_log(band_data.data)
+            res = _band_log(band_data.data,
+                            min_stretch[band_idx],
+                            max_stretch[band_idx])
             band_results.append(res)
 
         if 'A' in self.data.coords['bands'].values:
             band_results.append(self.data.sel(bands='A'))
-        self.data.data = da.stack(band_results,
-                                  axis=self.data.dims.index('bands'))
+        self.data.data = da.stack(band_results, axis=self.data.dims.index('bands'))
         self.data.attrs.setdefault('enhancement_history', []).append({'log_factor': factor})
 
+    def _convert_log_minmax_stretch(self, min_stretch, max_stretch):
+        non_band_dims = tuple(x for x in self.data.dims if x != 'bands')
+        if min_stretch is None:
+            min_stretch = [m.data for m in self.data.min(dim=non_band_dims)]
+        if max_stretch is None:
+            max_stretch = [m.data for m in self.data.max(dim=non_band_dims)]
+        if not isinstance(min_stretch, (list, tuple)):
+            min_stretch = [min_stretch] * self.data.sizes.get("bands", 1)
+        if not isinstance(max_stretch, (list, tuple)):
+            max_stretch = [max_stretch] * self.data.sizes.get("bands", 1)
+        return min_stretch, max_stretch
+
     def stretch_weber_fechner(self, k, s0):
         """Stretch according to the Weber-Fechner law.
 
